﻿/// <reference path="jquery-1.6.1.js"/> 


(function ($) {
    $.presence = function (options) {

        var settings = {
            'serviceUrl': '',
            'refreshRate': 4000,
            'statusImageBaseUrl': '', //a base url of a directory containing presense-sensitive pictures
            'statusImageExtension': '.jpg',
            'onPresenceItemUpdated': function () { }
        };

        if (options) {
            $.extend(settings, options);
        }

        if (settings.serviceUrl == "") {
            $.error("jquery.ucma.presence requires a service URL.");
        }

        var page = $("html");

        //Get a distinct list of sips on this page
        var sipElements = page.find('[class~="sip"]'); //get all sips
        var sips = new Array();
        $(sipElements).each(function (index, item) {   //for every sip..
            var sipValue = $.trim($(item).text());     //get the value..
            if ($.inArray(sipValue, sips) == -1) {     //and when the value is not yet in the array..
                sips.push(sipValue);                   //add it   
            }
        });

        //if a status image fails to load, show the unknown one instead
        $(".presenceitem").find('.statusImage').error(function () {
            $(this).stop().attr("src", settings.statusImageBaseUrl + "unknown.jpg").show();
        });

        //execute refreshPresence() every *settings.refreshRate* seconds
        page.everyTime(settings.refreshRate, function () {
            refreshPresence(settings, sips, page);
        });

        //force immediate execution (otherwise the system will wait *settings.refreshRate* seconds to execute the first time.)
        refreshPresence(settings, sips, page);
    };

    function refreshPresence(settings, sips, page) {

        $.ajax({
            type: 'GET',
            url: settings.serviceUrl + 'GetPresences?sipUris=' + sips + '&r=' + Math.random(),
            dataType: 'jsonp',
            success: function (presenceSubscriptions) {

                $.each(presenceSubscriptions, function (index, presenceSubscription) {

                    //each PresenceSubscription holds information about 1 sip

                    var sip = presenceSubscription.SipUri; //the sip of this PresenceSubscription

                    //get presenceitem containers for this sip on the page
                    var presenceitems = page.find(".sip:contains('" + sip + "')").parents(".presenceitem");

                    presenceitems.each(function () {

                        var presenceitem = $(this);

                        //add a class describing the availability to .availability elements
                        presenceitem.find('.availability').removeClass('availability unknown available offline away dnd busy').addClass(getTicTac(presenceSubscription.Availability));

                        //write the status into .status elements
                        if (presenceSubscription.Status !== "dnd") {
                            presenceitem.find('.status').html(presenceSubscription.Status);
                        } else {
                            presenceitem.find('.status').html("Do Not Disturb");
                        }

                        //write the email into .email elements
                        presenceitem.find('.email').html(presenceSubscription.Email);

                        //write the jobtitle into .jobtitle elements
                        presenceitem.find('.jobtitle').html(presenceSubscription.JobTitle);

                        //write the phone into .phone elements,
                        //skip skype'd elements because if its skyped, it means it has already been filled in
                        presenceitem.find('.phone:not(:has(.skype_pnh_print_container))').html(presenceSubscription.Phone); // todo: handle skype conflict with: .html("<a href='tel:" + presenceSubscription.Phone + "'>" + presenceSubscription.Phone + "</a>");

                        //write the mobile into .mobile elements
                        //skip skype'd elements because if its skyped, it means it has already been filled in
                        presenceitem.find('.mobile:not(:has(.skype_pnh_print_container))').html(presenceSubscription.Mobile);

                        //write the ContactName into .name elements
                        presenceitem.find('.name').html(presenceSubscription.ContactName);

                        //write the canChat into .canChat elements
                        presenceitem.find('.canChat').html(presenceSubscription.CanChat.toString());

                        if (settings.statusImageBaseUrl != '') {

                            //update an img.statusImage element if the image changed
                            var statusImage = presenceitem.find('.statusImage');

                            //build the new status image url
                            var statusFileName = getFilename(presenceSubscription.Status);
                            var statusImageHref = settings.statusImageBaseUrl + presenceSubscription.Email + "/" + statusFileName + settings.statusImageExtension;

                            // get the current status image url
                            var currentStatusImageHref = statusImage.attr("src");

                            if (currentStatusImageHref != statusImageHref) { //if they differ, update..

                                var browserVersion = parseInt($.browser.version, 10); //get the browser version

                                // ToDo: test this in ie 7 - 8, not sure if animation would work there

                                //if we run >ie8 or a decent browser, update the photo with a fancy animation
                                if ((!$.browser.msie || browserVersion > 8)
                                        && !endsWith(currentStatusImageHref, "unknown.jpg")) {  //dont use an animation if the previous picture is unknown.jpg, 
                                    //to avoid a hide/show loop for missing pictures)

                                    statusImage.stop().animate({ opacity: '0' }, function () {
                                        statusImage.show().attr('src', statusImageHref);
                                    }).load(function () {
                                        statusImage.stop().animate({ opacity: '1' });
                                    });
                                }
                                else {

                                    //if we run a failbrowser, update the photo without an animation
                                    statusImage.attr("src", statusImageHref).show();
                                }
                            }
                        }
                        settings.onPresenceItemUpdated.call(this, sip, presenceitem);
                    });
                });
            },
            error: function (xhr, ajaxOptions, thrownError) { }
        });
    }

    //extracts readable presence information based on the status
    function getTicTac(a) { var p = "availability unknown"; if (a < 3000) { presence = "availability unknown"; } else if (a < 4500) { p = "availability available"; } else if (a < 6000) { p = "availability away"; } else if (a < 7500) { p = "availability busy"; } else if (a < 9000) { p = "availability away"; } else if (a < 12000) { p = "availability dnd"; } else if (a < 18000) { p = "availability away"; } else { p = "availability offline"; } return p; }

    //converts status to photo filename
    function getFilename(a) { switch (a) { case "Do Not Disturb": return "dnd"; break; case "Online": return "available"; case "Available": return "available"; break; case "Offline": return "away"; break; case "Away": return "Away"; break; case "In a meeting": return "meeting"; break; case "Busy": return "busy"; break; case "On the phone": return "phone"; break; case "Be Right Back": return "away"; case "In a conference call": return "phone"; break; case "Unavailable": return "away"; break; default: return a; break } }

    function endsWith(text, value) {
        if (text == undefined) return false;
        return (text.match(value + "$") == value);
    }

})(jQuery);

