﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Text;
using log4net;
using Lync.Widget.Service.Common;
using Lync.Widget.Service.InstantMessaging;
using Lync.Widget.Service.Presence;
using Lync.Widget.Service.ServiceHelpers;

namespace Lync.Widget.Service
{
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class Service : IService
	{
		private static readonly ILog Log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

		private IPresenceCache _presenceCache;
		public static UcPresenceCache PresenceCache;

		public Service()
		{
			StartPresenceQueryService();

		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public PresenceSubscription GetPresence(string sipUri)
		{
			PresenceSubscription subscription = _presenceCache.GetPresence(sipUri);
			return subscription;
		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public List<PresenceSubscription> GetPresences(string sipUris)
		{
			if (sipUris == null) throw new ArgumentNullException("sipUris");

			var supscriptions = _presenceCache.GetPresence(sipUris.Split(',').ToList());

			return supscriptions;
		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public ServiceCallResponse InitChatSession(string session, string employeeSip, string webUserName, string page, string topic)
		{
			Log.Info(string.Format("Init chat requested from webUserName {0} for session {1} with employeeSip {2}, page {3}, topic {4}", webUserName, session, employeeSip, page, topic));
			try
			{
				UCInstantMessageRouter.Instance.InitChatSession(session, employeeSip, webUserName, page, topic);
			}
			catch (Exception e)
			{
				Log.Error(string.Format("Unable to init chat session {0}.", session), e);
				return ServiceCallResponseFactory.CreateFatalErrorResponse(e.Message);
			}
			return ServiceCallResponseFactory.CreateSuccesfulResponse();
		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public ServiceCallResponse SendInstantMessage(string session, string message)
		{
			Log.Info(string.Format("Message sent from webuser for session {0}: {1}", session, message));
			try
			{
				UCInstantMessageRouter.Instance.SendInstantMessageToLyncClient(session, message);
			}
			catch (Exception e)
			{
				Log.Error(string.Format("Unable to send message from webuser for session {0}.", session), e);
				return ServiceCallResponseFactory.CreateFatalErrorResponse(e.Message);
			}
			return ServiceCallResponseFactory.CreateSuccesfulResponse();
		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public ServiceCallResponse GetMessages(string session)
		{
			List<Message> messages;
			Log.Info(string.Format("webuser requests instant messages for session {0}", session));
			try
			{
				messages = UCInstantMessageRouter.Instance.GetInstantMessagesFromLynClient(session);
			}
			catch (Exception e)
			{
				Log.Error(string.Format("Unable to retrieve instant messages from queue for session {0}.", session), e);
				return ServiceCallResponseFactory.CreateFatalErrorResponse(e.Message);
			}
			return ServiceCallResponseFactory.CreateSuccesfulResponse(messages);
		}

		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public string LogStatusReport()
		{
			var statusReport = UCInstantMessageRouter.Instance.GetStatusReport();
			
			Log.Info("Status Report:");
			string answer = "Status Report: ";
			

			foreach (var property in typeof(StatusReport).GetProperties())
			{
				Log.Info("- " + property.Name + ": " + property.GetValue(statusReport, null));
				answer += "- " + property.Name + ": " + property.GetValue(statusReport, null);
			}

			return answer;
		}

		/// <summary>
		/// This method was implemented so that our sys admins are notified 
		/// when https://lyncservice.orbitone.com/service.svc/GetServiceStatus goes down.
		/// </summary>
		/// <returns>The string "Service Status: Online" that our monitoring tools can test against.</returns>
		[WebInvoke(Method = "GET", ResponseFormat = WebMessageFormat.Json)]
		public string GetServiceStatus()
		{
			return "Service Status: Online";
		}

		private void StartPresenceQueryService()
		{
			log4net.Config.XmlConfigurator.Configure();

			Log.Info("LyncWidget service starting up");
			UcmaHelper.GetInstance().Start(
				ConfigurationManager.AppSettings["ApplicationName"],
				ConfigurationManager.AppSettings["ApplicationId"]);
			IPresenceProvider provider = new UcPresenceProvider(UcmaHelper.GetInstance().ApplicationEndpoint);
			if (provider.Start())
			{
				_presenceCache = new UcPresenceCache(provider);
				Log.Info("LyncWidget service started successfully");
			}
			else
			{
				Log.Info("LyncWidget service failed to start");
			}

			//Log.Info("LyncWidget service starting up");
			//var queryService = new PresenceQueryService.ServiceMock();
			//_presenceQueryServiceHost = new WebServiceHost(queryService);
			//_presenceQueryServiceHost.Open();
			//Log.Info("LyncWidget service started successfully");
		}
	}
}
